<?php

namespace Drupal\solr_health_check\Controller;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Class for managing health checks for the Solr server.
 */
class HealthCheckController extends ControllerBase {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a HealthCheckController object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(ConfigFactoryInterface $config_factory, EntityTypeManagerInterface $entity_type_manager) {
    $this->configFactory = $config_factory;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * Check Solr health for all indexes.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The JSON response indicating status for each index.
   */
  public function check() {
    $config = $this->configFactory->get('solr_health_check.settings');
    $return_500_on_error = $config->get('return_500_on_error');

    $statuses = [];
    $all_cores_up = TRUE;
    $index_storage = $this->entityTypeManager->getStorage('search_api_index');
    $indexes = $index_storage->loadMultiple();

    foreach ($indexes as $index) {
      /** @var \Drupal\search_api\IndexInterface $index */
      try {
        /** @var \Drupal\search_api_solr\Plugin\search_api\backend\SearchApiSolrBackend $backend */
        $backend = $index->getServerInstance()->getBackend();
        $solr_connector = $backend->getSolrConnector();
        $ping = $solr_connector->pingCore();

        if (is_float($ping)) {
          $statuses[$index->id()] = [
            'status' => 'up',
            'response_time' => $ping,
          ];
        }
        else {
          $all_cores_up = FALSE;
          $statuses[$index->id()] = [
            'status' => 'down',
            'message' => 'Solr responded but with an error status.',
            'error_status' => $ping->getStatus(),
            'error_message' => $ping->getResponse()->getStatusMessage(),
          ];
        }
      }
      catch (\Exception $e) {
        $all_cores_up = FALSE;
        $statuses[$index->id()] = [
          'status' => 'down',
          'message' => $e->getMessage(),
        ];
      }
    }

    $response = new JsonResponse($statuses);
    if ($return_500_on_error && !$all_cores_up) {
      $response->setStatusCode(500);
    }
    else {
      $response->setStatusCode(200);
    }

    $response->headers->set('Cache-Control', 'no-cache, no-store, must-revalidate');
    $response->headers->set('Pragma', 'no-cache');
    $response->headers->set('Expires', '0');

    return $response;
  }

}
